@@ -1,17 +1,25 @@
 // Copyright 2021 Vladislav Melnik
 // SPDX-License-Identifier: MIT
 
-use super::{proposal::Proposal, request::Request};
+use super::{
+    proposal::{Proposal, ReadOnce, WriteOnce},
+    request::Request,
+};
 
 /// The deterministic state machine.
 /// It accepts proposals from network and issues requests.
 /// All business logic should be implemented inside.
-pub trait State {
+pub trait State<R, W>
+where
+    R: ReadOnce,
+    W: WriteOnce,
+{
     /// Some user defined inputs to the state machine
-    type ProposalExt;
+    type Ext;
+
+    /// Randomness needed by the state machine
+    type Rng;
 
     /// In order to preserve determinism, it should be the only input to the state machine.
-    fn accept_proposal<R>(&mut self, proposal: Proposal<R, Self::ProposalExt>) -> Request
-    where
-        R: rand::Rng;
+    fn accept(&mut self, proposal: Proposal<R, W, Self::Ext, Self::Rng>) -> Request;
 }
